"""Plot source space NDVars with mayavi/pysurfer."""
# Author: Christian Brodbeck <christianbrodbeck@nyu.edu>

from ._brain import (activation, cluster, dspm, stat, surfer_brain, annot,
                     bin_table, copy, image, p_map, annot_legend)
